# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import filters, permissions, serializers

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# PROFILE
class ProfileViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    ProfileViewSet
    --------------

    :action list: Return the profile list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given profile.
        `Profiles`: `superuser`, `staff`

    :action light: Return the profile list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new profile instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a profile instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a profile instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a profile instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Profile.objects.all()
    serializer_class = serializers.ProfileSerializer
    permission_classes = (permissions.ProfilePermission,)
    filter_backends = (filters.ProfileFilterBackend,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a profile instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a profile instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ROLE
class RoleViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    RoleViewSet
    -----------

    :action list: Return the role list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given role.
        `Profiles`: `superuser`, `staff`

    :action light: Return the role list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new role instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a role instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a role instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a role instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Role.objects.all()
    serializer_class = serializers.RoleSerializer
    permission_classes = (permissions.RolePermission,)
    filter_backends = (filters.RoleFilterBackend,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a role instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a role instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# MENU
class MenuViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    MenuViewSet
    -----------

    :action list: Return the menu list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given menu.
        `Profiles`: `superuser`, `staff`

    :action light: Return the menu list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new menu instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a menu instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a menu instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a menu instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Menu.objects.all()
    serializer_class = serializers.MenuSerializer
    permission_classes = (permissions.MenuPermission,)
    filter_backends = (filters.MenuFilterBackend,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a menu instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a menu instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# MENU TRANSLATION
class MenuTranslationViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    MenuTranslationViewSet
    -----------

    :action list: Return the menu translation list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given menu translation.
        `Profiles`: `superuser`, `staff`

    :action light: Return the menu translation list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new menu translation instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a menu translation instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a menu translation instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a menu translation instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.MenuTranslation.objects.all()
    serializer_class = serializers.MenuTranslationSerializer
    permission_classes = (permissions.MenuTranslationPermission,)
    filter_backends = (filters.MenuTranslationFilterBackend,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a menu translation instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a menu translation instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
