# -*- coding: utf-8 -*-
import importlib
from typing import Optional

from ..utils import ApiRole


class ThuxRoleViewMixin:
    """Mixin for views in the thx-role Django application.

    This mixin is responsible for initializing the `ApiRole` instance and writing logs.

    Attributes:
        _ar (Optional[ApiRole]): The `ApiRole` instance associated with the view.
    """

    _ar: Optional[ApiRole] = None

    @property
    def ar(self) -> ApiRole:
        """
        ApiRole property. Needed for views inspector tools like thx-openapi.
        These tools don't always call the `initial` method.
        """
        if self._ar:
            return self._ar
        self.init_ar(self.request, self, *self.args, **self.kwargs)
        return self._ar

    def init_ar(self, request, *args, **kwargs):
        """Initialize the `ApiRole` instance.

        This method creates an `ApiRole` instance using the provided `request`, the view instance (`self`), and any
        additional positional and keyword arguments.

        :param request: The request object.
        :param args: Variable length argument list.
        :param kwargs: Arbitrary keyword arguments.
        """
        self._ar = ApiRole(request, self, *args, **kwargs)

    def initial(self, request, *args, **kwargs):
        """Initialize the `ApiRole` instance and call the parent class's `initial` method.

        :param request: The request object.
        :param args: Variable length argument list.
        :param kwargs: Arbitrary keyword arguments.
        """
        self.init_ar(request, *args, **kwargs)
        super().initial(request, *args, **kwargs)
        self.ar.write_log()

    def get_serializer_class(self):
        """Get the serializer class for the view.

        If a custom serializer class is specified in the associated `ApiRole` instance, it is returned.
        Otherwise, the default serializer class is returned.

        :return: rest_framework.serializers.Serializer: The serializer class for the viewset.
        """
        serializer = self.ar.get_serializer_class()
        if serializer:
            return serializer
        return super().get_serializer_class()


class ThuxRoleViewSetMixin(ThuxRoleViewMixin):
    """Mixin for viewsets in the thx-role Django application.

    This mixin extends the `ThuxRoleViewMixin` to provide additional functionality for determining the serializer class
    based on the associated `ApiRole` instance.

    """

    def get_serializer_class(self):
        """Get the serializer class for the viewset.

        If a custom serializer class is specified in the associated `ApiRole` instance, it is returned. Otherwise, the
        serializer class is determined based on the action and model name. If a serializer class with the name
        "{ModelName}{Action}Serializer" is found in the same module as the viewset's serializer class, it is returned.
        If no serializer is found, the default serializer class is returned.

        :return: rest_framework.serializers.Serializer: The serializer class for the viewset.
        """
        serializer = self.ar.get_serializer_class()
        if serializer:
            return serializer
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        model_name = self.queryset.model.__name__
        serializer = getattr(
            importlib.import_module(self.serializer_class.__module__), f'{model_name}{action}Serializer', None
        )
        if serializer:
            return serializer
        return super().get_serializer_class()
